import functools
from abc import ABC
from inspect import signature, Parameter
from typing import Optional

from cpl_query.extension import List


class FilterABC(ABC):
    def __init__(self):
        ABC.__init__(self)

        self._page_index = None
        self._page_size = None
        self._sort_direction = None
        self._sort_column = None

    @property
    def page_index(self) -> Optional[int]:
        return self._page_index

    @property
    def page_size(self) -> Optional[int]:
        return self._page_size

    @property
    def sort_direction(self) -> Optional[str]:
        return self._sort_direction

    @property
    def sort_column(self) -> Optional[str]:
        return self._sort_column

    def skip_and_take(self, query: List):
        if self._page_size is not None and self.page_index is not None:
            skip = self.page_size * self.page_index
            result = query.skip(skip).take(self.page_size)
            return result

        return query

    @staticmethod
    def get_filter(f, values: dict):
        sig = signature(f)
        for param in sig.parameters.items():
            parameter = param[1]
            if parameter.name == "self" or parameter.name == "cls" or parameter.annotation == Parameter.empty:
                continue

            if issubclass(parameter.annotation, FilterABC):
                filter = parameter.annotation()
                filter.from_dict(values)
                return filter

    @classmethod
    def resolve_filter_annotation(cls, f=None):
        if f is None:
            return functools.partial(cls.resolve_filter_annotation)

        @functools.wraps(f)
        def decorator(*args, **kwargs):
            if "filter" in kwargs:
                kwargs["filter"] = cls.get_filter(f, kwargs["filter"])

            return f(*args, **kwargs)

        return decorator
