from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.level import Level
from bot_graphql.abc.query_abc import QueryABC


class LevelMutation(QueryABC):
    def __init__(self, servers: ServerRepositoryABC, levels: LevelRepositoryABC):
        QueryABC.__init__(self, "LevelMutation")

        self._servers = servers
        self._levels = levels

        self.set_field("create_level", self.resolve_create_level)
        self.set_field("update_level", self.resolve_create_level)
        self.set_field("delete_level", self.resolve_create_level)

    def resolve_create_level(self, *_, input: dict):
        level = Level(
            input["name"],
            input["color"],
            int(input["min_xp"]),
            int(input["permissions"]),
            self._servers.get_server_by_id(input["server_id"]),
        )
        return level

    def resolve_update_level(self, *_, input):
        return self._levels.get_level_by_id(input.id)

    def resolve_delete_level(self, *_, id: int):
        return self._levels.get_level_by_id(id)
