from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.model.server import Server
from bot_graphql.abc.data_query_abc import DataQueryABC
from bot_graphql.abc.filter_abc import FilterABC
from bot_graphql.filter.level_filter import LevelFilter


class ServerQuery(DataQueryABC):
    def __init__(
        self,
        bot: DiscordBotServiceABC,
        levels: LevelRepositoryABC,
    ):
        DataQueryABC.__init__(self, "Server")

        self._bot = bot
        self._levels = levels

        self.set_field("id", self.resolve_id)
        self.set_field("discord_id", self.resolve_discord_id)
        self.set_field("name", self.resolve_name)
        self.set_field("levels", self.resolve_levels)

    @staticmethod
    def resolve_id(server: Server, *_):
        return server.server_id

    @staticmethod
    def resolve_discord_id(server: Server, *_):
        return server.discord_server_id

    def resolve_name(self, server: Server, *_):
        guild = self._bot.get_guild(server.discord_server_id)
        return None if guild is None else guild.name

    @FilterABC.resolve_filter_annotation
    def resolve_levels(self, server: Server, *_, filter: LevelFilter = None):
        if filter is not None:
            return filter.filter(self._levels.get_levels_by_server_id(server.server_id))

        return self._levels.get_levels_by_server_id(server.server_id)
